/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siscli.h"
#if 0
#include "ccs.h"
#endif
#include "sislogger.h"

inline void mutually_exclusive(char *progname, char flag1, char flag2)
{
	fprintf(stderr, "%s: flag '%c' conflicts with '%c'.\n",
		progname, flag2, flag1);
	fprintf(stderr, "See man page for details.\n");
}

/**
 * ccs_initlog - initialize the ccs defaults from the ccs file
 * @filename: ccs file to retrieve info from
 * @nodepath: full path to node to search in
 * @nodename: node to look for key values in
 * @verbose:  user set verbosity (-1 to use ccs value)
 * @quiet:    user set quiet (-1 to use ccs value)
 *
 * Description: attempts to read the log init values from ccs.  the integer
 * values default to 0 and the character strings default to NULL if they aren't
 * found in ccs.
 *
 * Returns: zero on success, non-zero on error
 */
int ccs_initlog(char *filename, char *nodename, int verb, int qt)
{
#if 1
        return -1;
#else
	int error = 0;
	ccs_node_t *root_cn = NULL;
	ccs_node_t *cn = NULL;
	int syslog;
	int verbose;
	int debug;
	int quiet;
	const char *msg_prefix;
	const char *logfile;
	char sep = '/';
	char *nodepath = NULL;
        char *data = NULL;
        char *fn = NULL;
        int count = 0;
        int fexists = 0;

        /* Check to see if the ccs file exists first */
        error = get_ccs_file(&data, "list_files");

        if(error != 0)
                goto end;

        fn = filename;
        while(((count = strcspn(data, "\n")) != 0) &&
              (count <= strlen(data))) {
                if(strlen(filename) == count) {
                        if(strncmp(data, filename, count) == 0) {
                                fexists = 1;
                                break;
                        }
                }
                count++;
                fn = filename + count;
                data += count;
        }

        if(fexists != 1) {
                error = -ENOENT;
                goto end;
        }

	error = open_ccs_file(&root_cn, filename);
	if(error) {
		/* Failure here might mean CCS isn't loaded, or the file
		   isn't available, or you don't have permission to access
		   the file...we don't know...*/
		error = -1;
		goto end;
	}

	nodepath = (char *) malloc((strlen(nodename) +
				    strlen("syslog")+2)*sizeof(char));
	if(nodepath == NULL) {
		close_ccs_file(root_cn);
		error = ENOMEM;
		goto end;
	}
	strcpy(nodepath, nodename);
	strcat(nodepath, "/");
	strcat(nodepath, "syslog");

	if((cn = find_ccs_node(root_cn, nodepath, sep)) == NULL) {
		/* likely this error is caused by the nodename not being there,
		   but it's possible other error conditions exist */
		error = -2;
		goto end;
	}

	free(nodepath);
	nodepath = NULL;

	syslog = find_ccs_int(cn, "syslog", sep, 0);
	debug = find_ccs_int(cn,  "debug", sep, LOG_WARN);
	msg_prefix = find_ccs_str(cn, "log_prefix", sep, NULL);
	logfile = find_ccs_str(cn, "logfile", sep, NULL);
	if(verb < 0)
		verbose = find_ccs_int(cn, "verbose", sep, 0);
	else
		verbose = verb;
	if(qt < 0)
		quiet = find_ccs_int(cn, "quiet", sep, 0);
	else
		quiet = qt;

	error = init_sislog(verbose, quiet, debug, syslog,
			    msg_prefix, logfile);

	close_ccs_file(root_cn);

 end:
	return error;
#endif
}
